import ilp_solving, heuristic_solving
from pre_processing import *
import time
from post_processing import *


def solve_time_windows_ilp(airport, flightset, passengers, alpha, beta, use_greedy):
    # Set the initial time window
    t_start = START_AIRPORT_DISRUPTION_TIME
    t_end = t_start + WINDOW_DURATION
    solution_status = 'optimal'
    max_time = 0
    computation_time_list = []

    # Initialize tabu lists for greedy heuristic
    if use_greedy:
        list_tabu_dep_flights = []
        dico_tabu_arr_flights = heuristic_solving.initialized_tabu_arr_flights(flightset, passengers)

    while (t_start + WINDOW_SHIFT <= T_MAX and solution_status == 'optimal'):
        # Set the status of flights within the current time window
        set_status_flights(flightset, airport, t_start, t_end)

        # Compute terminal occupancy
        compute_terminals_occupancy(flightset, airport)

        print(
            f"********************Run problem between {t_start * TIME_STEP // 60}min and {t_end * TIME_STEP // 60}min************************")
        start_time = time.time()

        if use_greedy:
            # Run the greedy heuristic
            flights_a, flights_d, list_tabu_dep_flights, dico_tabu_arr_flights, computational_time = heuristic_solving.run_greedy(
                airport, flightset,
                passengers, t_start, t_end,
                list_tabu_dep_flights,
                dico_tabu_arr_flights)
        else:
            # Run the ILP solver
            flights_a, flights_d, solution_status, computational_time = ilp_solving.ilp_pfrsp(airport, flightset,
                                                                                               passengers, alpha, beta,
                                                                                               t_start,
                                                                                               t_end)

        computation_time = np.round(time.time() - start_time, 3)

        if computation_time > max_time:
            max_time = computation_time

        computation_time_list.append(computation_time)
        print(
            f'Computation time between {t_start * TIME_STEP // 60}min and {t_end * TIME_STEP // 60}:  {computation_time}s')

        # Update the flightset with the obtained solution
        flightset = update_flightset(flightset, flights_a, flights_d)
        t_start += WINDOW_SHIFT
        t_end += WINDOW_SHIFT

    if (use_greedy == False) and (solution_status == 'optimal'):
        print('************ OPTIMIZATION RUN FOR THE FULL DAY ************')

    print(f"Computation time per time window (s) : {computation_time_list}")
    return flightset


def main(use_greedy=False, savefig=False):
    # Read input instance
    airport, flightset, passengers = read_instance()
    start = time.time()

    # Solve the ILP problem for time windows
    flightset_ilp = solve_time_windows_ilp(airport, flightset, passengers, ALPHA, BETA, use_greedy)
    write_solution(flightset_ilp, label='')
    computational_time = time.time() - start
    print(f" Total computational time: {round(computational_time, 2)}s")

    # Perform post-processing and evaluate results
    tot_stranded_pax_bo, tot_stranded_pax_ao, tot_deviation_dep_flights, tot_deviation_arr_flights, tot_slots_violated = post_processing(
        airport, flightset_ilp, passengers, ALPHA, BETA, test_constraint=True,
        terminal_step_constraint=TERMINAL_STEP_CONSTRAINT, display_constraints=True,
        taxi_step_constraint=TAXI_STEP_CONSTRAINT, runway_step_constraint=RUNWAY_STEP_CONSTRAINT,
        savefig=savefig, is_greedy=use_greedy)

    return tot_stranded_pax_bo, tot_stranded_pax_ao, tot_deviation_arr_flights, tot_deviation_dep_flights, tot_slots_violated


# Run the main function with specified parameters
main(use_greedy=False, savefig=True) # if savefig = False, display them
